@@ -162,10 +162,6 @@ static GtkActionEntry action_entries[] = {
 	  NULL, "<control>A",
 	  N_("Select all files"),
 	  G_CALLBACK (activate_action_select_all) },
-	{ "Stop", GTK_STOCK_STOP,
-	  NULL, "Escape",
-	  N_("Stop current operation"),
-	  G_CALLBACK (activate_action_stop) },
 	{ "TestArchive", NULL,
 	  N_("_Test Integrity"), NULL,
 	  N_("Test whether the archive contains errors"),
